﻿<%@ WebService Language="C#" Class="ResourceFinder" %>
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Web.Script.Services;
using System.Data.Common;
using System.Collections;
using System.Text;

/// <summary>
/// Find resources based on a given string of text
/// </summary>
[ScriptService]
public class ResourceFinder : System.Web.Services.WebService {
    /// <summary>
    /// Handles AJAX requests to find resources.
    /// </summary>
    /// <param name="language">The language of the translation entered.</param>
    /// <param name="resource">The translation to match against.</param>
    /// <returns>A List of languages/resources/comments</returns>
    [ScriptMethod(ResponseFormat = ResponseFormat.Json)]
    [WebMethod]
    public Hashtable FindResource(string language, string translation, int offset) {
        List<Resource> resourceIds = new List<Resource>();
        int limit = 5,
            found = 0;
        Database db = new Database();
        db.OpenConnection();
        using(Database.Command cmd = db.CreateCommand()) {
            StringBuilder sqlBuffer = new StringBuilder();
            switch(db.InstanceType) {
                case Database.DatabaseType.MySQL:
                    sqlBuffer.Append("SELECT r.resourceid, translation, comment FROM ").Append(db.TablePrefix).Append("resources AS r LEFT JOIN ").Append(db.TablePrefix)
                             .Append("resourcecomments AS c ON r.resourceid = c.resourceid WHERE language = @language AND translation LIKE @translation ORDER BY translation LIMIT ")
                             .Append(limit).Append(" OFFSET ").Append(offset);
                    break;
                case Database.DatabaseType.SQLServer:
                    int pageNum = (offset / limit) + 1;
                    sqlBuffer.Append(@";WITH results AS (
                                    SELECT r.resourceid, translation, comment, rowNo = ROW_NUMBER() OVER( ORDER BY translation ASC) 
                                    FROM ").Append(db.TablePrefix).Append("resources AS r LEFT JOIN ").Append(db.TablePrefix).Append(@"resourcecomments AS c ON r.resourceid = c.resourceid 
                                    WHERE language = @language AND translation LIKE @translation
                                ) SELECT * FROM results
                                WHERE rowNo BETWEEN (").AppendFormat(" {0}-1) * {1}+1 AND {0} * {1}",pageNum,limit);
                    break;
            }
            cmd.CommandText = sqlBuffer.ToString();
            cmd.AddParameterWithValue("@language", HttpUtility.UrlDecode(language));
            cmd.AddParameterWithValue("@translation", '%' + HttpUtility.UrlDecode(translation) + '%');
            DbDataReader reader = cmd.ExecuteReader();
            while(reader.Read()) {
                resourceIds.Add(new Resource(HttpUtility.UrlEncode(reader[0].ToString()), HttpUtility.UrlEncode(reader[1].ToString()), HttpUtility.UrlEncode(reader[2].ToString())));
            }
            reader.Close();
            cmd.CommandText = "SELECT COUNT(resourceid) FROM " + db.TablePrefix + "resources WHERE language = @language AND translation LIKE @translation";
            found = int.Parse(cmd.ExecuteScalar().ToString());
        }
        db.CloseConnection();
        Hashtable results = new Hashtable();
        results.Add("resources", resourceIds);
        results.Add("found", found);
        results.Add("perpage", limit);
        results.Add("offset", offset);
        results.Add("language", language); // could get this from the ID of the text box but it's easier/safer to pass it back to JS
        //foreach(KeyValuePair<string, object> s in resourceIds) {System.Diagnostics.Debug.Write(s+"-"+s.Value.);
        return results;
    }

    /// <summary>
    /// A class used by the resource finder service which can hold resource data
    /// </summary>
    public class Resource {
        public string ID { get; set; }
        public string Text { get; set; }
        public string Comment { get; set; }

        public Resource(string id, string text, string comment) {
            this.ID = id;
            this.Text = text;
            this.Comment = comment;
        }
    }

    /// <summary>
    /// Finds resources matching an ID.
    /// </summary>
    /// <param name="resourceid">The resource ID to match.</param>
    /// <returns>A key/value collection of languages/resources</returns>
    public static SortedList<string, string> FindResource(string resourceid) {
        SortedList<string, string> resources = new SortedList<string, string>();
        Database db = new Database();
        db.OpenConnection();
        using(Database.Command cmd = db.CreateCommand()) {
            cmd.CommandText = "SELECT language, translation FROM " + db.TablePrefix + "resources WHERE resourceid = @tID";
            cmd.AddParameterWithValue("@tID", resourceid);
            DbDataReader reader = cmd.ExecuteReader();
            while(reader.Read()) {
                resources.Add(reader[0].ToString(), reader[1].ToString());
            }
        }
        db.CloseConnection();
        return resources;
    }
    
}

